#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from pathlib import Path
import wx
import time
import datetime as dt
from typing import Optional, List, Tuple
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
from nardascripting.base.measthread import MeasDlgThread
from . import Vita49_Streaming_Client_Python as V49


def load_settings(file_name: str):
    """ Loads the settings from a file in the current directory and returns them as dictionary.

    :param file_name: The name of the file.
    :return: A dictionary containing the settings.
    """
    json_str = ""
    settings_path = Path(__file__).parent.joinpath(file_name)

    if settings_path.exists():
        json_str = settings_path.read_text()
        return json.loads(json_str)
    else:
        return None


def record_iq_as_wav(stopevent, update_progress,
                     signalshark: SignalSharkDev, settings: {}, ftune: float, cbw: float) -> Optional[str]:
    """ Switches to RT Streaming mode and records the IQ stream in a WAV file.

    This Python function should be used together with a NardaScriptLauncher MeasDlgThread!
    Please do NOT use any wxPython elements like a MessageBox or a Dialog here!
    They are only permitted in the GUI thread ('_run_script' method).
    Please use the "update_progress" method to display information!

    :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
    :param update_progress: Delegate method to update a progress bar, message string and icon
    :param signalshark: The SignalShark device
    :param settings: The iq recorder settings
    :param ftune: The Ftune frequency of the IQ stream in Hz.
    :param cbw: The CBW of the IQ stream in Hz.
    :return: If an error occurs, an error message is returned otherwise the return value is None.
    """

    # Setup connection to the SignalShark and display error message if connection fails.
    if not signalshark.connect():
        return "Cannot connect to the device!"

    err_msg = None
    try:
        con_addr = ""

        # Setup file names:
        # -------------------------------------------------------------------------------
        time_stamp = dt.datetime.now()
        base_path = settings.get("dirpck_file_path", r"c:\\")
        filename_prefix = settings.get("txt_filename_prefix", "IQ_Data")
        wav_file_name = time_stamp.strftime(filename_prefix + "_%Y-%m-%d_%H-%M-%S.wav")
        wav_file_path = Path(base_path).joinpath(wav_file_name)
        txt_file_name = time_stamp.strftime(filename_prefix + "_%Y-%m-%d_%H-%M-%S.txt")
        txt_file_path =  Path(base_path).joinpath(txt_file_name)

        # Define communication and VITA49 settings for use with DLL:
        # -------------------------------------------------------------------------------
        baseSettings = V49.Vita49_Base_Settings()
        baseSettings.strIpAddress = settings.get("txt_device_ip", "127.0.0.1")
        baseSettings.strAdapterIp = settings.get("txt_adapter_ip", "127.0.0.1")
        conn_type = settings.get("cbx_conn_type", "UDP_SINGLECAST")
        if conn_type == "TCP_SERVER":
            baseSettings.eConnectionType = V49.PARAM_CONNECTION_TYPE.eTCP_SERVER
            con_addr = baseSettings.strIpAddress
        elif conn_type == "TCP_CLIENT":
            baseSettings.eConnectionType = V49.PARAM_CONNECTION_TYPE.eTCP_CLIENT
            con_addr = baseSettings.strIpAddress
        elif conn_type == "UDP_SINGLECAST":
            baseSettings.eConnectionType = V49.PARAM_CONNECTION_TYPE.eUDP_SINGLECAST
            con_addr = baseSettings.strAdapterIp
        else:
            baseSettings.eConnectionType = V49.PARAM_CONNECTION_TYPE.eUDP_MULTICAST
            con_addr = baseSettings.strAdapterIp
        baseSettings.unPortTCP = settings.get("sctr_port", 4444)
        baseSettings.unPortUDPSrc = settings.get("sctr_port", 4444)
        baseSettings.ulStreamID = settings.get("sctr_stream_id", 12345)

        # Define VITA49 receiver settings:
        receiverSettings = V49.Vita49_Receiver_Settings()
        receiverSettings.bDataBigEndian = False
        receiverSettings.bAccessData = False
        receiverSettings.bRawFileBigEndian = False
        receiverSettings.strRawFileName = ""
        receiverSettings.bWaveFilePCMFormat = True
        receiverSettings.strWaveFileName = str(wav_file_path)

        # Readout current measurement settings:
        # -------------------------------------------------------------------------------
        task_type, task_name = signalshark.scpi.task.get_selected()
        rf_input = signalshark.scpi.sense.get_input()
        atten = signalshark.scpi.sense.get_attenuator()

        # Setup IQ stream:
        # -------------------------------------------------------------------------------
        if not signalshark.scpi.check_add_task(TaskTypes.RT_STREAMING, "IQRecStream"):
            signalshark.scpi.stream.add(StrTypes.VITA49_IQ)
            signalshark.scpi.stream.set_select(1)
        else:
            signalshark.scpi.stream.set_select(1)

        con_state = signalshark.scpi.stream.get_connection_state()
        if con_state == StrConStates.CONNECTED or con_state == StrConStates.OPENED:
            signalshark.scpi.stream.connection_close()

        signalshark.scpi.stream.set_connection_idn(settings.get("sctr_stream_id", 12345))
        signalshark.scpi.stream.set_connection_type(StrConTypes.from_string(settings.get("cbx_conn_type", "UDP_SINGLECAST")))
        signalshark.scpi.stream.set_connection_address(con_addr)
        signalshark.scpi.stream.set_connection_port(settings.get("sctr_port", 4444))

        signalshark.scpi.iqstream.set_oversampling(settings.get("cbx_oversampling", False))
        signalshark.scpi.iqstream.set_payload_limit(settings.get("cbx_payload_limit", "2048"))
        signalshark.scpi.iqstream.set_payload_format(StrPayForm.from_string(settings.get("cbx_payload_format", "I16Q16")))

        # Signal settings
        signalshark.scpi.sense.set_input(rf_input)
        signalshark.scpi.sense.set_attenuator(atten)
        signalshark.scpi.iqstream.set_frequency_tune(ftune)
        signalshark.scpi.iqstream.set_cbw(cbw)
        signalshark.scpi.stream.connection_open()

        with open(str(txt_file_path), "w") as f:
            f.write("Measurement: " + filename_prefix + "\n")
            f.write("Timestamp: " + time_stamp.strftime("%Y-%m-%d %H:%M:%S") + "\n")
            f.write("Input: " + str(rf_input) + "\n")
            f.write("Atten: " + str(atten) + "dB\n")
            f.write("Ftune: " + str(ftune/1e6) + " MHz\n")
            f.write("CBW: " + str(cbw/1e6) + " MHz\n")

        if signalshark.scpi.check_error():
            return "Error while stream initialization."

        # Record VITA49 IQ data as wav-file:
        # -------------------------------------------------------------------------------
        vita49Receiver = V49.Vita49Receiver()
        vita49Receiver.Init(baseSettings, receiverSettings)

        # connectionErrorCallback = lambda: print("Connection error")
        # vita49Receiver.SetConnectionErrorCallback(connectionErrorCallback)

        eRet = vita49Receiver.Open()
        seconds = settings.get("sctr_rec_seconds", 6)
        cnt = 0
        progress = 0.0
        update_progress(msg="Recording IQ data", progress=0.0)
        while not stopevent.is_set() and cnt < seconds:
            update_progress(progress=progress)
            cnt += 1
            progress = cnt/seconds
            stopevent.wait(1)
        eRet = vita49Receiver.Close()

        if not stopevent.is_set():
            update_progress(msg="Recording done", progress=1.0)

        signalshark.scpi.task.set_select(task_name)
        signalshark.scpi.stream.connection_close()
        time.sleep(0.2)

    except Exception as e:
        # Do some error handling
        err_msg = str(e)
    return err_msg


# This is just a python module containing helper functions:
if __name__ == '__main__':
    pass