#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


from pathlib import Path
import wx
import time
import datetime
from nardascripting.base.usrscriptbase import *
from nardascripting.base.signalsharkdev import *
from nardascripting.base.measthread import MeasDlgThread
from .gui.thresholddlg import ThresholdDlg
from . import iqrecorder as IQRec

SETTINGS_FILENAME = "iqrec_settings.json"

class IQRecorder(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = "nsliqrecorder"
        self._scr_title = "IQ Recorder by Peak Table V0.9"
        self._scr_description = "The Ftune parameter is determined by the signal of the " \
                                "peak table with the highest level."
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")
        self._list_prio = 1
        self._nsl_executed_behavior = NSL_Executed_Behaviors.SHOW_NSL

    def _run_script(self, args):
        """ Script main function

        This method is called when a user clicks on the corresponding script button.
        """
        # Load settings
        # -------------------------------------------------------------------------------
        self.settings = IQRec.load_settings(SETTINGS_FILENAME)
        if not self.settings:
            self.MessageBoxModal("Please run settings script first!",
                                 "Coverage Map", wx.OK | wx.ICON_INFORMATION)
            return

        # Connect to the device
        # -------------------------------------------------------------------------------
        if not self.signalshark.connect():
            self.MessageBoxModal("Can not connect to device!", "Connection error",
                                 wx.OK | wx.ICON_ERROR)
            return

        self.signalshark.scpi.check_error()

        # Add a RT Spectrum Task if it does not already exist.
        if not self.signalshark.scpi.check_add_task(TaskTypes.RT_SPECTRUM):
            self.MessageBoxModal("Please configure the newly added task (e.g. frequency range)", "New Task",
                                 wx.OK | wx.ICON_INFORMATION)
            self.signalshark.disconnect()
            return

        task_type, task_name = self.signalshark.scpi.task.get_selected()
        if task_type != TaskTypes.RT_SPECTRUM:
            self.MessageBoxModal("Please select the RT Spectrum Task first!",
                                 "Select Task",
                                 wx.OK | wx.ICON_INFORMATION)
            self.signalshark.disconnect()
            return

        # Setup trigger via peak table:
        # -------------------------------------------------------------------------------

        # Configure the task regarding the trigger function
        self.signalshark.scpi.spectrum.set_trace_enable(SpecTraceTypes.MxP, True)

        # Add a Peak Table to monitor the spectrum if it does not already exist and configure it
        exists, view_index = self.signalshark.scpi.check_add_view(ViewTypes.PEAK_TABLE)
        if view_index < 1:
            self.MessageBoxModal("Adding peak table not possible.",
                                 "Peak Table", wx.OK | wx.ICON_ERROR)
            return

        if not exists:
            self.signalshark.scpi.display.set_peaktable_sort(PeakSort.LEVEL)
            self.signalshark.scpi.marker.set_spectrum_search_peak_excursion(3.0)
            self.signalshark.scpi.marker.set_spectrum_search_peak_excursion_enable(True)

        # Get trigger level:
        # -------------------------------------------------------------------------------
        sig_thr = 0
        unit = self.signalshark.scpi.display.get_unit()
        curr_thr = self.signalshark.scpi.marker.get_spectrum_search_threshold()
        dlg = ThresholdDlg(self.frm_dlghelper, "Threshold:", curr_thr, str(unit))
        if self.ShowDlgModalTop(dlg) == wx.ID_OK:
            sig_thr = dlg.GetValue()
        else:
            dlg.Destroy()  # Please do not forget to destroy the created dialog if you do not need it anymore
            # User has canceled selection
            return
        dlg.Destroy()  # Please do not forget to destroy the created dialog if you do not need it anymore

        # Configure the Peak Table for the new trigger level
        self.signalshark.scpi.marker.set_spectrum_search_threshold(sig_thr)
        self.signalshark.scpi.marker.set_spectrum_search_limits_enable(True)

        # Run measurement acquisition
        task_state = self.signalshark.scpi.task.get_state()
        if task_state != TaskStates.RUN:
            self.signalshark.scpi.sense.run_continuous()

        # Disconnect from the device
        self.signalshark.disconnect()


        # Create an additional measurement thread to keep the GUI alive during measurement.
        # -------------------------------------------------------------------------------
        # Please change 'MyUserScript._run_measurement' according to your class name.
        # With "args=[param1, parm2, paramn]" you can pass some optional parameters to the measurement thread.
        mthread = MeasDlgThread(main_gui=self.main_gui, callback=IQRecorder._run_measurement,
                                args=[self.signalshark.addr, self.settings])
        # Start the measurement thread and show a visualization (progress) dialog.
        mthread.start_measurement()

        # Disconnect from the device
        self.signalshark.disconnect()


    @staticmethod
    def _run_measurement(stopevent, update_progress, wait_for_stopevent,
                         signalshark_addr: str, settings: {}):
        """ Main method of the measurement thread that handles a time consuming measurement.

        Please do NOT use any wxPython elements like a MessageBox or a Dialog here!
        They are only permitted in the GUI thread ('_run_script' method).
        Please use the "update_progress" method to display information!

        :param stopevent: Thread flag that indicates whether the procedure should be finished/cancelled
        :param update_progress: Delegate method to update a progress bar, message string and icon
        :param wait_for_stopevent: Delegate method to wait until stopevent is raised with possibility to show a message.
        :param signalshark_addr: Custom parameter to handover the ip address of the SignalShark
        :param settings: The iq recorder settings
        :return:
        """

        # Setup connection to the SignalShark and display error message if connection fails.
        signalshark = SignalSharkDev(signalshark_addr)
        if not signalshark.connect():
            return wait_for_stopevent("Cannot connect to the device!", icon_style=wx.ICON_ERROR)

        try:
            # Wait until a signal has exceeded the trigger level or the user wants to cancel the process.
            # -------------------------------------------------------------------------------
            # Update the dialog message text and show a progress bar
            update_progress(msg="Waiting for trigger", show_progress=True)
            signalshark.scpi.run_cont()
            ftune = 0.0
            while True:
                # Periodically check, whether the user has clicked the cancel button,
                # and exit thread if so.
                if stopevent.is_set():
                    return

                # Wait until Scan Number and Peak Table data count have been increased.
                scan_number = signalshark.scpi.peaktable.get_data_update()
                if scan_number > 0:
                    signalshark.scpi.hold()
                    data_cnt = signalshark.scpi.peaktable.get_data_count()
                    if data_cnt > 0:
                        # Get signal frequency
                        freq_str_arr = signalshark.scpi.peaktable.get_data_frequency()
                        ftune = float(freq_str_arr[0])
                        break

                # Allow the thread to synchronize the stop event.
                stopevent.wait(0.05)

            if stopevent.is_set():
                update_progress(msg="Measurement aborted")
                time.sleep(1)
                return

            # Call IQ recorder function.
            # -------------------------------------------------------------------------------
            cbw = settings.get("sctrd_cbw", 3) * 1e6
            err_msg = IQRec.record_iq_as_wav(stopevent, update_progress,
                                            signalshark, settings, ftune, cbw)
            if err_msg:
                wait_for_stopevent(msg=err_msg, icon_style=wx.ICON_ERROR)
        except Exception as e:
            # Do some error handling
            return wait_for_stopevent(str(e), icon_style=wx.ICON_ERROR)
        finally:
            # Close connection.
            signalshark.disconnect()
            time.sleep(0.5)