#!/usr/bin/env python

##############################################################################
##
## Copyright (C) 2021 Narda Safety Test Solutions GmbH.
## Contact: www.narda-sts.com
##
## Redistribution and use in source and binary forms,
## with or without modification, are permitted provided that the following
## conditions are met:
##
## 1.) Redistributions of source code must retain the above copyright notice,
## this list of conditions and the following disclaimer.
##
## 2.) Redistributions in binary form must reproduce the above copyright
## notice, this list of conditions and the following disclaimer in the
## documentation and/or other materials provided with the distribution.
##
## 3.) Neither the name of the copyright holder nor the names of its
## contributors may be used to endorse or promote products derived from this
## software without specific prior written permission.
##
## 4.) EXPORT CONTROL
## The Software, including technical data /cryptographic software,
## may be subject to, German, European Union and U.S. export controls and
## may be subject to import or export controls in other countries.
## The Licensee agrees to strictly comply with all applicable import and
## export regulations. He specifically agrees, that he must not disclose or
## otherwise export or re-export the Licensed Software or any part thereof
## delivered under this end user license agreement (EULA) to any country
## (including a national or resident of such country) without a valid export
## or import license. Please be aware that the Software may contain
## US-Content, therefor the Licensee represent and warrant that he is not a
## citizen of, or otherwise located within, an embargoed nation (including
## without limitation Iran, Syria, Sudan, Cuba, North Korea) and that he is
## not otherwise prohibited under the Export Laws from receiving the Software.
## All rights to Use the Software are granted on condition that such rights
## are forfeited if the Licensee fails to comply with
## the terms of this Agreement.
##
## 5.) SEVERABILITY
## Should any provision of this Agreement be or become invalid, ineffective
## or unenforceable, the remaining provisions of this Agreement shall be
## valid. The parties agree to replace the invalid, ineffective or
## unenforceable provision by a valid, effective and enforceable provision
## which best meets the commercial intention of the parties.
## The same shall apply in case of omissions.
##
## 6.) APPLICABLE LAW AND PLACE OF JURISTICATION
## This Agreement shall be constituted under the law of Germany.
## The United Nations Convention on the International Sale of Goods
## shall not apply to this Agreement. The Place of Jurisdiction for any
## dispute between the Parties shall be Tübingen (Germany).
##
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
## “AS IS” AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
## TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
## PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Narda Safety Test Solutions GmbH
## BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
## CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
## SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
## INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
## CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
## ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
## POSSIBILITY OF SUCH DAMAGE.
##
##############################################################################


import json
from pathlib import Path

from nardascripting.base.signalsharkdev import *
from nardascripting.base.usrscriptbase import *
import nardascripting.base.toolbox as tb
from .gui.iqrecdlgs import IQRecSettingsDlgBase


SETTINGS_FILENAME = "iqrec_settings.json"


class IQRecSettings(UsrScriptBase):
    """User script class"""

    def __init__(self, main_gui, dev=SignalSharkDev()):
        """Initialization. Please leave this line unchanged"""
        super().__init__(main_gui, dev, __file__)

        # Script settings.
        # -------------------------------------------------------------------------------
        # Please adapt the following lines of code according to your script.
        self._tab_name = "nsliqrecorder"
        self._scr_title = "IQ Recorder Settings 0.9"
        self._scr_description = "Settings for the IQ Recorder script"
        # self._icon_path = self.script_path.joinpath("NardaScriptLauncher_example01.png")
        self._list_prio = 3
        self._nsl_executed_behavior = NSL_Executed_Behaviors.SHOW_NSL

    def _run_script(self, args):
        """
        Script main function
        This method is called when a user clicks on the corresponding script button.
        """
        dlg = IQRecSettingsDlg(self.frm_dlghelper)
        self.ShowDlgModalTop(dlg)
        dlg.Destroy()

class IQRecSettingsDlg(IQRecSettingsDlgBase):
    """Custom dialog class for iq recorder settings."""

    def __init__(self, parent):
        """Dialog class for data visualization.

        :param parent: The parent window as wx.Window.
        """
        super().__init__(parent=parent)

        # init gui:
        self._init_controls()

        # load settings:
        self.settings = {}
        self._load_settings()

    def _init_controls(self):
        # Connection types:
        for item in StrConTypes:
            value = str(item)
            if value == "UNKNOWN":
                continue
            self.cbx_conn_type.Append(value)

        # cbx_payload_limit
        self.cbx_payload_limit.AppendItems(["256","512","1024","2048"])

        # Payload formats:
        for item in StrPayForm:
            value = str(item)
            if value == "UNKNOWN":
                continue
            self.cbx_payload_format.Append(value)

    def _load_settings(self):
        """Loads the coverage map settings from file"""
        json_str = ""
        self.settings_path = Path(__file__).parent.joinpath(SETTINGS_FILENAME)

        if self.settings_path.exists():
            json_str = self.settings_path.read_text()
            self.settings = json.loads(json_str)

        # Rec. Settings
        self.sctr_rec_seconds.SetValue(self.settings.get("sctr_rec_seconds", 6))
        self.sctrd_cbw.SetValue(self.settings.get("sctrd_cbw", 3))
        self.dirpck_file_path.SetPath(self.settings.get("dirpck_file_path", r"c:\\"))
        self.txt_filename_prefix.SetValue(self.settings.get("txt_filename_prefix", "IQ_Data"))

        # Con. Settings
        self.cbx_conn_type.SetValue(self.settings.get("cbx_conn_type", "UDP_SINGLECAST"))
        self.txt_device_ip.SetValue(self.settings.get("txt_device_ip", "127.0.0.1"))
        self.txt_adapter_ip.SetValue(self.settings.get("txt_adapter_ip", "127.0.0.1"))
        self.sctr_port.SetValue(self.settings.get("sctr_port", 4444))
        self.sctr_stream_id.SetValue(self.settings.get("sctr_stream_id", 12345))

        # Stream Settings
        self.cbx_oversampling.SetValue(self.settings.get("cbx_oversampling", False))
        self.cbx_payload_limit.SetValue(self.settings.get("cbx_payload_limit", "1024"))
        self.cbx_payload_format.SetValue(self.settings.get("cbx_payload_format", "I16Q16"))

    def _save_settings(self):
        """Saves the site depending device settings from file"""
        # Rec. Settings
        self.settings["sctr_rec_seconds"] = self.sctr_rec_seconds.GetValue()
        self.settings["sctrd_cbw"] = self.sctrd_cbw.GetValue()
        self.settings["dirpck_file_path"] =self.dirpck_file_path.GetPath()
        self.settings["txt_filename_prefix"] = self.txt_filename_prefix.GetValue()

        # Con. Settings
        self.settings["cbx_conn_type"] = self.cbx_conn_type.GetValue()
        self.settings["txt_device_ip"] = self.txt_device_ip.GetValue()
        self.settings["txt_adapter_ip"] = self.txt_adapter_ip.GetValue()
        self.settings["sctr_port"] = self.sctr_port.GetValue()
        self.settings["sctr_stream_id"] = self.sctr_stream_id.GetValue()

        # Stream Settings
        self.settings["cbx_oversampling"] = self.cbx_oversampling.GetValue()
        self.settings["cbx_payload_limit"] = self.cbx_payload_limit.GetValue()
        self.settings["cbx_payload_format"] = self.cbx_payload_format.GetValue()

        json_str = ""
        store_path = Path(__file__).parent.joinpath(SETTINGS_FILENAME)
        json_str = json.dumps(self.settings, indent=2, sort_keys=True)
        store_path.write_text(json_str)

    def on_ok_button_click(self, event):
        self._save_settings()
        event.Skip()
